# ************************************************************
# @(#) web/webserver/cgi/Core/Form/Input.pm web_r3.6.2.3.1:cset.001111:22:33@(#)
#
#   Filename    : web/webserver/cgi/Core/Form/Input.pm
#
# *************************************************************
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                       Vallent
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# Copyright (c) 2005 Vallent Software Systems UK. All rights reserved.
#
# **************************************************************
#
#   Description : Input class - DON'T even think of touching
#                 this - happy coding
#
# ************************************************************
#
package Core::Form::Input ;

# set up logging
require "$ENV{'NPRWEB_DIR'}/cgi/Lib/Logger.pm";
my $logger = new Logger( $ENV{NPRWEB_NPR_LOGFILE}, $ENV{NPRWEB_NPR_LOGLEVEL} );

##################################################################################
#
# Function    : new()
#
# Parameters  : MW_system - hash of form variables,
#
# Description : Constructor for report/graph form
#
# Returns     : object reference
#
# Called By   : Core::Form->display()
#
##################################################################################
sub new {
    my $proto = shift;    # get the class name
    my $class = ref($proto) || $proto;
    # if class exists, use it else use reference.
    my $this = {MW_system => {},
                @_} ;

    $this->{tabWidth} = 0 ;
    $this->{tabLeft} = 0 ;

    bless $this, $class;
    return $this ;

}

##################################################################################
#
# Function    : header()
#
# Parameters  : $header - header
#
# Description : generates a html tagged title
#
# Returns     : html string
#
# Called By   : ??::Form->displaySelect(),
#               ??::CounterBrowsers::Form->display()
#
##################################################################################
sub header {
    my ($this, $header) = @_ ;

    return qq{<table width="540">
<tr >
<td class="reportTitle" colspan="3" align="center" height="30">
<p class="bluehead1">$header</p>
</td>
</tr>
</table>
    } ;
}

##################################################################################
#
# Function    : tab()
#
# Parameters  : $tabNumber - tab number,
#               $tabInfo - tab information,
#               $onClickInfo - javascript onClick information
#
# Description : generates a html tagged tab
#
# Returns     : html string
#
# Called By   : ??::Form->displaySelect(),
#               ??::CounterBrowsers::Form->display()
#
##################################################################################
sub tab {
    my ($this, $tabNumber, $tabInfo, $onClickInfo) = @_ ;
    my $tab = "" ;

    # left image
    $tab .= qq{
<div id="tab_$tabNumber" style="position:absolute;top:0px;left:5px;cursor:hand;" onclick="${onClickInfo}" selected="$$tabInfo{selected}">
<div style="position:absolute;top:55px;left:$this->{tabLeft}px;width:6px;height:22px;"><img src="/images/tabs/curved_tab_left.gif" /></div>
    } ;

    my $tabLength = length($$tabInfo{name})*8 ; ;

    # text
    $this->{tabLeft} += 6 ;
    $tab .= qq{
<div style="position:absolute;left:$this->{tabLeft}px;top:57px;width:${tabLength}px;height:19px;font-family:tahoma;font-size:8pt;text-align:center;vertical-align:text-bottom;color:white;background-color:#3366cc;visibility:visible;">$$tabInfo{name}</div>
    } ;

    # calculate tabWidth
    $this->{tabWidth} += $tabLength ;

    # right image
    $this->{tabLeft} = $this->{tabWidth} + 6;
    $tab .= qq{
<div style="position:absolute;top:55px;left:$this->{tabLeft}px;width:6px;height:22px;"><img src="/images/tabs/curved_tab_right.gif" /></div>
    } ;

    # div end
    $tab .= qq{
</div>
    };

    # calculate tabWidth
    $this->{tabWidth} += 12 ;
    $this->{tabLeft} += 6 ;

    return $tab ;
}

##################################################################################
#
# Function    : panel()
#
# Parameters  : $tabNumber - tab number,
#               $tabInfo - tab information,
#
# Description : generates html tagged panel of elements for the gui
#
# Returns     : html string
#
# Called By   : ??::Form->displaySelect()
#
##################################################################################
sub panel {
    my ($this, $tabNumber, $tabInfo ) = @_ ;
    my $pane = "" ;

    $pane .= qq{
    <div id="panel_$tabNumber" class="panel" selected="$$tabInfo{selected}">
        <table width="495">
        <tr><td colspan='4'></td></tr>
    } ;
    # handle hidden data items
    $pane .= $this->_hiddenNPRVars($tabNumber, $$tabInfo{fields}) ;

    # handle database items
    $pane .= $this->_dataBaseItems($tabNumber, $$tabInfo{fields}) ;

    # handle data items
    $pane .= $this->_NPRVars($tabNumber, $$tabInfo{fields}) ;

    $pane .= qq{
        </table>
    </div>
    } ;

    return $pane ;
}

##################################################################################
#
# Function    : rmSelect()
#
# Parameters  : none
#
# Description : generates a html tagged selection list for the data retrieval method
#
# Returns     : html string
#
# Called By   : ??::Form->displaySelect()
#
##################################################################################
sub rmSelect {
    my ($this, $display) = @_ ;
    my $dialog = "" ;

    if ( uc($display) eq 'DATABASE' ) {
        $dialog .= qq[
<td caption="retrievalDB" valign="top" class="panelText">Data retrieval method:</td>
<td style="color:white" align="left">NPR database </td>
<input id="RM" name="RM" type="hidden" value="DATABASE">
        ];
    } elsif ( uc($display) eq 'CACHE' ) {
        $dialog .= qq[
<td caption="retrievalDB" valign="top" class="panelText">Data retrieval method:</td>
<td style="color:white" align="left">Hierarchy cache </td>
<input id="RM" name="RM" type="hidden" value="CACHE">
        ];
    } else {
        $dialog .= qq[
<td caption="retrieval" valign="top" class="panelText">Data retrieval method:</td>
<td align="left">
<select id="RM" name="RM" onFocus="hilight(this)" onBlur="lolight(this);" size="1" onchange="Refresh_Screen(this);">
<option value="CACHE"
        ];

        if(!exists $this->{MW_system}{RM} || $this->{MW_system}{RM} eq 'CACHE') {
            $dialog .= qq[selected>Hierarchy cache</option>\n];
        } else {
            $dialog .= qq[>Hierarchy cache</option>\n];
        }

        $dialog .= qq[<option value="DATABASE"\n];

        if(exists $this->{MW_system}{RM} && $this->{MW_system}{RM} eq 'DATABASE') {
            $dialog .= qq[selected>NPR database</option>\n];
        } else {
            $dialog .= qq[>NPR database</option>\n];
        }
        $dialog .= qq[
</select>
</td>
        ];
    }
    return $dialog ;
}

##################################################################################
#
# Function    : Private - _select()
#
# Parameters  : $dataItems - hash of data items relating to $entryField
#
# Description : generates select input tag for select entry fields
#
# Returns     : html string
#
# Called By   : _dataItems()
#
##################################################################################
sub _select {
    my ($this, $dataItems) = @_ ;

    $dialog = qq{   <select class="inputs" size="1" onFocus="hilight(this)" onBlur="lolight(this);" name="$$dataItems{TslVarName}">\n} ;

    foreach $selectDataArray ($$dataItems{SelectData}) {
	foreach $selectDataAttributes (@{$selectDataArray}) {
	    $dialog .= qq[   <option value="$$selectDataAttributes{SelectValue}"];

	    # Pre-selected the summary period if defined
	    if ( $$selectDataAttributes{SelectValue} eq $this->{MW_system}{SUMMARY} ) {
		$dialog .= " selected";
	    }	
	    $dialog .= qq[>$$selectDataAttributes{SelectName}</option>\n];
	}
    }
    $dialog .= qq{   </select>\n};

    return $dialog ;
}

##################################################################################
#
# Function    : Private - _date()
#
# Parameters  : $dataItems - hash of data items relating to $entryField
#
# Description : generates text date input tag for text date entry fields
#
# Returns     : html string
#
# Called By   : _dataItems()
#
##################################################################################
sub _date {
    my ($this, $dataItems) = @_ ;
    my $dialog = "" ;

    if ( $$dataItems{DisplayLabel} eq 'Target Date' and $$dataItems{TslVarName} eq 'DAY' ) {
        $dialog .= qq[
<input type="hidden" name="TARGETD" value="$this->{MW_system}{TARGETD}" />
<input type="text" value="$this->{MW_system}{TARGETD}" size="9" maxlength="9" caption="$$dataItems{DateFormat}$this->{MW_system}{DATEFORMAT}" class="inputs" id="$$dataItems{TslVarName}" name="$$dataItems{TslVarName}" onFocus="hilight(this)" onBlur="lolight(this);" />
        ] ;
	$dialog .= $this->_calendarButton($$dataItems{TslVarName}) ;
    } elsif ( $$dataItems{DisplayName} =~ 'Target' and $$dataItems{TslVarName} eq 'DAY' ) {
        $dialog .= qq[
<input type="text" size="9" maxlength="9" caption="$$dataItems{DateFormat}$this->{MW_system}{DATEFORMAT}" id="$$dataItems{TslVarName}" class="inputs" name="$$dataItems{TslVarName}" value="$this->{MW_system}{TARGETD}" onFocus="hilight(this)" onBlur="lolight(this);">
	    ] ;
	$dialog .= $this->_calendarButton($$dataItems{TslVarName}) ;
    } elsif ( $$dataItems{DisplayName} =~ 'Start') {
         $dialog .= qq[<input type="text" size="9" maxlength="9" caption="$$dataItems{DateFormat}$this->{MW_system}{DATEFORMAT}" id="$$dataItems{TslVarName}" class="inputs" name="$$dataItems{TslVarName}" value="$this->{MW_system}{STARTD}" onFocus="hilight(this)" onBlur="lolight(this);">
         ] ;
         $dialog .= $this->_calendarButton($$dataItems{TslVarName}) ;
    } elsif ( $$dataItems{DisplayName} =~ 'End') {
         $dialog .= qq[<input type="text" size="9" maxlength="9" caption="$$dataItems{DateFormat}$this->{MW_system}{DATEFORMAT}" id="$$dataItems{TslVarName}" class="inputs" name="$$dataItems{TslVarName}" value="$this->{MW_system}{ENDD}" onFocus="hilight(this)" onBlur="lolight(this);">
         ] ;
         $dialog .= $this->_calendarButton($$dataItems{TslVarName}) ;
    }  else {
        $dialog .= qq[<input type="text" size="9" maxlength="9" caption="$$dataItems{DateFormat}$this->{MW_system}{DATEFORMAT}" id="$$dataItems{TslVarName}" class="inputs" name="$$dataItems{TslVarName}" value="\$\$$$dataItems{TslVarName}\$\$" onFocus="hilight(this)" onBlur="lolight(this);">
            ] ;
        $dialog .= $this->_calendarButton($$dataItems{TslVarName}) ;
    }

    return $dialog ;

}

##################################################################################
#
# Function    : Private - _text()
#
# Parameters  : $dataItems - hash of data items relating to $entryField
#
# Description : generates text input tag for text entry fields
#
# Returns     : html string
#
# Called By   : _dataItems()
#
##################################################################################
sub _text {
    my ($this, $dataItems) = @_ ;

    return qq{
<input class="inputs" type="text" size="30" name="$$dataItems{TslVarName}" value="\$\$$$dataItems{TslVarName}\$\$" onFocus="hilight(this)" onBlur="lolight(this);">
    } ;
}

##################################################################################
#
# Function    : Private - _check()
#
# Parameters  : $dataItems - hash of data items relating to $entryField
#
# Description : generates check input tag for check entry fields
#
# Returns     : html string
#
# Called By   : _dataItems()
#
##################################################################################
sub _check {
    my ($this, $dataItems) = @_ ;

    return qq{
<input class="inputs" type="checkbox" size="30" name="$$dataItems{TslVarName}" value="\$\$$$dataItems{TslVarName}\$\$" onFocus="hilight(this)" onBlur="lolight(this);">
    } ;
}

##################################################################################
#
# Function    : Private - _calendarButton()
#
# Parameters  : $inputName - reference name when returning calendar value
#
# Description : generates html tagged calendar button
#
# Returns     : html string
#
# Called By   : _date()
#
##################################################################################
sub _calendarButton {
    my ($this, $inputName) = @_ ;

	# add Calendar button
    return qq[
<input type="image" style="vertical-align:middle;" src="/images/buttons/calendar.gif" value="Calendar" caption="calendar" onclick="window.open('/cgi/Date/calendar.pl?NPRSYS=$this->{MW_system}{NPRSYS}&amp;input=$inputName', 'Calendar', 'width=220,height=220'); return false;" />
    ] ;
}

##################################################################################
#
# Function    : Private - _dataBaseItems()
#
# Parameters  : $tabNumber - scalar tab number,
#               $tabInfo - hash of tab information,
#
# Description : generates html tagged database items for use in the gui
#
# Returns     : html string
#
# Called By   : display()
#
##################################################################################
sub _dataBaseItems {
	my ($this, $tabNumber, $tabInfo) = @_ ;
	my $dialog = "" ;
	my $dataBaseItems = $this->{dataBaseItems} ;

	my $includeOOA;
	if ( $this->{MW_system}{USE_OOA} eq 'true' ) {
		$logger->log( "info - USE_OOA is true", 4 );
		$includeOOA = "return OnlyOneAllowed(this.value, this.name);";
	}
	
	foreach (@$tabInfo ) {
		if ( exists( $$dataBaseItems{$_}{Name} ) ) {   # not necsessary to have TabNumber container
			# use substitute name if it exists
			if ( $$dataBaseItems{$_}{SubDBName} ne "" ) {
				$value= uc( $$dataBaseItems{$_}{SubDBName} ) ;
			} else {
				$value= uc( $$dataBaseItems{$_}{DBName} ) ;
			}
			$dbName = uc( $$dataBaseItems{$_}{DBName} ) ;
			
			my $fld_filled;
			foreach my $fld (@$tabInfo) {
				#if ($$dataBaseItems{$fld}) {
				# BUG 57284
				if ($$dataBaseItems{$fld}{DBName}) {
					my $fldDBName = uc( $$dataBaseItems{$fld}{DBName} ) ;
					$fld_filled .= $fldDBName.":";
				}
			}
			chop($fld_filled);
			 
			# valnt00102641 : COUNTER BROWSER ELEMENT SLOW IN FILTERING (APAR=IZ69191)
			$dialog .= qq{
				<tr>
				<td class="panelText">$$dataBaseItems{$_}{DisplayName}</td>
				<td colspan="2">
			};
			
			if ($ENV{ELEMENT_SUGGESTION} eq "False") {
				$dialog .= qq{
					<input id="$dbName" techlayer="$this->{MW_system}{TECH_LAYER}" nprsys="$this->{MW_system}{NPRSYS}" class="networkElement" length=70 type="text" dbItem='$_' name="$dbName" value="\$\$$value\$\$" onFocus="hilight(this)" onBlur="lolight(this);$includeOOA"></td>
				};
			}else{
				$logger->log("\$ENV{ELEMENT_SUGGESTION} is " . $ENV{ELEMENT_SUGGESTION},4);
				my $divID = $dbName."Div";
				$dialog .= qq{
					<input id="$dbName" techlayer="$this->{MW_system}{TECH_LAYER}" nprsys="$this->{MW_system}{NPRSYS}" class="networkElement" length=70 type="text" dbItem='$_' name="$dbName" value="\$\$$value\$\$" maxSuggest="$ENV{ELEMENT_SUGGESTION}" onFocus="hilight(this)" onkeyup="doSuggestion(this);return false;" onBlur="lolight(this);$includeOOA">
					<br><div id="$divID" visibility="hidden" position="relative"></div>
					</td>
				};
			}
			
			$dialog .= qq{
					<td>
				<input type="image" src="/images/buttons/drop_arrow.gif" value="list" onclick="_viewElementList(this);return false;" name="$$dataBaseItems{$_}{Name}" currlist="$fld_filled" href="$this->{MW_system}{CGIPROG}?NPRSYS=$this->{MW_system}{NPRSYS}&amp;NPRTARGET=frm_Select.htm&amp;FIRSTPASS=true&amp;CURRSELECT=$$dataBaseItems{$_}{Name}&amp;CURRSELECTBY="></td>
				</tr>
			} ;
			# Get the name of each box for use in OnlyOneAllowed
			push @{$this->{disp_dataBaseTypes}}, $$dataBaseItems{$_}{DBName} ;
		}
	}
	return $dialog ;
}

##################################################################################
#
# Function    : Private - _NPRVars()
#
# Parameters  : $tabNumber - scalar tab number,
#               $tabInfo - hash of tab information,
#
# Description : generates html tagged data items for use in the gui
#
# Returns     : html string
#
# Called By   : display()
#
##################################################################################
sub _NPRVars {
    my ($this, $tabNumber, $tabInfo) = @_ ;
    my $dialog = "" ;
    my $NPRVars = $this->{NPRVars} ;

    #$logger->log("info - IN: _NPRVars()",4);

    foreach (@$tabInfo ) {
        #$logger->log("info - processing '$_'",4);
	    if ( exists( $$NPRVars{$_}{Name} ) ) {   # not necsessary to have TabNumber container
            $dialog .= qq{
<tr><td class="panelText">$$NPRVars{$_}{DisplayName}</td>
    <td>\n};

    	    # the data item is a select box
    	    if ( $$NPRVars{$_}{InputType} eq 'select' ) {
                $dialog .= $this->_select($$NPRVars{$_});

    	    # the data item is a text box
    	    } elsif ( $$NPRVars{$_}{InputType} eq 'text' ) {
                if ( $$NPRVars{$_}{TslVarType} eq 'date' ) {
       		    $dialog .= $this->_date($$NPRVars{$_});
                } else {
       		    $dialog .= $this->_text($$NPRVars{$_});
                }

    	    # the data item is a check box
    	    } elsif ( $$NPRVars{$_}{InputType} eq 'check' ) {
                $dialog .= $this->_check($$NPRVars{$_});

    	    # unrecognised Data Entry
            } else {
                $dialog .= qq{
<p style="color:orange">Core::Form - Unrecognised Data Entry Field '$_' in in_file_di!</p>
    		    } ;
            }
            $dialog .= qq[   </td></tr>];
        }
    }
    #$logger->log("info - OUT: _NPRVars()",4);

    return $dialog ;
}

##################################################################################
#
# Function    : Private - _hiddenNPRVars()
#
# Parameters  : $tabNumber - scalar tab number,
#               $tabInfo - hash of tab information,
#
# Description : generates html tagged hidden data items for use in the gui
#
# Returns     : html string
#
# Called By   : display()
#
##################################################################################
sub _hiddenNPRVars {
    my ($this, $tabNumber, $tabInfo) = @_ ;
    my $hiddenNPRVars = $this->{hiddenNPRVars} ;
    my $dialog = "" ;
    foreach (@$tabInfo ) {
	    if ( exists( $$hiddenNPRVars{$_}{Name} ) ) {   # not necsessary to have TabNumber container
            $hiddenName = uc ( $$hiddenNPRVars{$_}{Name} );
            $hiddenValue = uc ( $$hiddenNPRVars{$_}{Value} ) ;
            $dialog .= qq[<input type=hidden name="$hiddenName" value="\$\$$hiddenValue\$\$" />] ;
        }
    }
    return $dialog ;
}

sub DESTROY {

    my ($this) = @_ ;
    undef $this->{dataBaseItems};
    undef $this->{NPRVars};
    undef $this->{hiddenNPRVars};

}

##################################################################################
#
# Function    : _onlyOneAllowed()
#
# Parameters  : none
#
# Description : generates a html tagged javascript function which allows only one
#               element to be selected
#
# Returns     : html string
#
# Called By   : Form::Display->displaySelect()
#
##################################################################################
sub _onlyOneAllowed {
    my ($this,$disp_dataBaseTypes, $display) = @_ ;

    return "" if uc($display) eq 'NO' ;

    my $element;
    my $dialog = "" ;

    # OnlyOneAllowd:  For each element box the inner foreach script is created
    # so that the box that has just been written to remains the same and the
    # rest are blank.

    $dialog .= qq[<scr] . qq[ipt language='javascript'>\n] ;
    $dialog .= qq[function OnlyOneAllowed(Value, Name) {\n ] ;

    foreach $element (@{$disp_dataBaseTypes}) {
        $element = uc($element);
        #$logger->log("info - \$element = $element",4) ;
        $dialog .= qq[
if (Name == '$element') {
//alert ('they are the same')
document.nprform1.$element.value= Value
} else {
//alert('they are different')
document.nprform1.$element.value = ''
}
        ] ;
    }

    $dialog .= qq[
}
    ] ;

    $dialog .= qq[</scr] . qq[ipt>\n];

    return $dialog ;
}

1;
