#!/bin/sh
set -a

clear

# Check that required files exist within the package
echo "\nChecking required files to apply patch... \c"
for fl in WEB3623.UpgradeList WEB3623.Source.tar
do
	if [ ! -f $fl ]
	then
		echo "FAILED.\n\tRequired '$fl' is missing from the tarball.\n"
		exit
	fi
done
echo "PASSED."

# Check that the the IDS installation directory is defined 
echo "\nChecking for a valid Metrica/WEB installation directory (\$NPRWEB_DIR)... \c"
if [ ! -f $NPRWEB_DIR/cgi/webnpr3 ]
then
	echo "FAILED.\n\tMissing '$NPRWEB_DIR/cgi/webnpr3'. Please check the path.\n"
	exit
fi
echo "PASSED."

# Locating files that need to be changed
echo "\nLocating files... \c"
for fl in `cat WEB3623.UpgradeList`
do
	if [ ! -f $NPRWEB_DIR/$fl ]
	then
		echo "MISSING '$fl'. \n\tExecution will be terminated."
		exit
	fi
done
echo "ALL FOUND."

currdir=`pwd`

# Create backup for all files that need to be changed
echo "\nBacking up files... \c"
cd $NPRWEB_DIR
for fl in `cat $currdir/WEB3623.UpgradeList`
do
	mv $fl $fl.preIF3
done
echo "DONE."

echo "\nApplying patch WEB3623.Source.tar... \c"
tar xpf $currdir/WEB3623.Source.tar
echo "COMPLETED.\n"

# Finally go back to patching directory
cd $currdir
